@@ -89,8 +89,8 @@ static void PrintUsageAndDie() {
   P("Options for HashDBM:\n");
   P("  --append : Uses the appending mode rather than the in-place mode.\n");
   P("  --record_crc num : The record CRC mode: -1, 0, 8, 16, 32. (default: 0 or -1)\n");
-  P("  --record_comp str : The record compression mode: default, none, zlib, zstd, lz4, lzma."
-    " (default: none or default)\n");
+  P("  --record_comp str : The record compression mode:"
+    " default, none, zlib, zstd, lz4, lzma, rc4, aes. (default: none or default)\n");
   P("  --offset_width num : The width to represent the offset of records. (default: %d)\n",
     HashDBM::DEFAULT_OFFSET_WIDTH);
   P("  --align_pow num : Sets the power to align records. (default: %d)\n",
@@ -101,12 +101,13 @@ static void PrintUsageAndDie() {
     HashDBM::DEFAULT_FBP_CAPACITY);
   P("  --min_read_size num : Sets the minimum reading size to read a record. (default: -1)\n");
   P("  --cache_buckets : Caches the hash buckets on memory.\n");
+  P("  --cipher_key str : Sets the encryption key for cipher compressors. (default: empty)\n");
   P("\n");
   P("Options for TreeDBM and FileIndex:\n");
   P("  --append : Uses the appending mode rather than the in-place mode.\n");
   P("  --record_crc num : The record CRC mode: -1, 0, 8, 16, 32. (default: 0 or -1)\n");
-  P("  --record_comp str : The record compression mode: default, none, zlib, zstd, lz4, lzma."
-    " (default: none or default)\n");
+  P("  --record_comp str : The record compression mode:"
+    " default, none, zlib, zstd, lz4, lzma, rc4, aes. (default: none or default)\n");
   P("  --offset_width num : The width to represent the offset of records. (default: %d)\n",
     TreeDBM::DEFAULT_OFFSET_WIDTH);
   P("  --align_pow num : Sets the power to align records. (default: %d)\n",
@@ -117,6 +118,7 @@ static void PrintUsageAndDie() {
     TreeDBM::DEFAULT_FBP_CAPACITY);
   P("  --min_read_size num : Sets the minimum reading size to read a record. (default: -1)\n");
   P("  --cache_buckets : Caches the hash buckets on memory.\n");
+  P("  --cipher_key str : Sets the encryption key for cipher compressors. (default: empty)\n");
   P("  --max_page_size num : Sets the maximum size of a page. (default: %d)\n",
     TreeDBM::DEFAULT_MAX_PAGE_SIZE);
   P("  --max_branches num : Sets the maximum number of branches of inner nodes. (default: %d)\n",
@@ -230,7 +232,8 @@ bool SetUpDBM(DBM* dbm, bool writable, bool initialize, const std::string& file_
               bool with_no_wait, bool with_no_lock, bool with_sync_hard,
               bool is_append, int32_t record_crc, const std::string& record_comp,
               int32_t offset_width, int32_t align_pow, int64_t num_buckets,
-              int32_t fbp_cap, int32_t min_read_size, bool cache_buckets,
+              int32_t fbp_cap, int32_t min_read_size,
+              bool cache_buckets, const std::string cipher_key,
               int32_t max_page_size, int32_t max_branches, int32_t max_cached_pages,
               bool page_update_write, int32_t step_unit, int32_t max_level,
               int64_t sort_mem_size, bool insert_in_order, int32_t max_cached_records,
@@ -274,6 +277,10 @@ bool SetUpDBM(DBM* dbm, bool writable, bool initialize, const std::string& file_
       tuning_params.record_comp_mode = tkrzw::HashDBM::RECORD_COMP_LZ4;
     } else if (record_comp == "lzma") {
       tuning_params.record_comp_mode = tkrzw::HashDBM::RECORD_COMP_LZMA;
+    } else if (record_comp == "rc4") {
+      tuning_params.record_comp_mode = tkrzw::HashDBM::RECORD_COMP_RC4;
+    } else if (record_comp == "aes") {
+      tuning_params.record_comp_mode = tkrzw::HashDBM::RECORD_COMP_AES;
     }
     tuning_params.offset_width = offset_width;
     tuning_params.align_pow = align_pow;
@@ -282,6 +289,7 @@ bool SetUpDBM(DBM* dbm, bool writable, bool initialize, const std::string& file_
     tuning_params.fbp_capacity = fbp_cap;
     tuning_params.min_read_size = min_read_size;
     tuning_params.cache_buckets = cache_buckets;
+    tuning_params.cipher_key = cipher_key;
     const Status status =
         hash_dbm->OpenAdvanced(file_path, writable, open_options, tuning_params);
     if (status != Status::SUCCESS) {
@@ -313,6 +321,10 @@ bool SetUpDBM(DBM* dbm, bool writable, bool initialize, const std::string& file_
       tuning_params.record_comp_mode = tkrzw::HashDBM::RECORD_COMP_LZ4;
     } else if (record_comp == "lzma") {
       tuning_params.record_comp_mode = tkrzw::HashDBM::RECORD_COMP_LZMA;
+    } else if (record_comp == "rc4") {
+      tuning_params.record_comp_mode = tkrzw::HashDBM::RECORD_COMP_RC4;
+    } else if (record_comp == "aes") {
+      tuning_params.record_comp_mode = tkrzw::HashDBM::RECORD_COMP_AES;
     }
     tuning_params.offset_width = offset_width;
     tuning_params.align_pow = align_pow;
@@ -321,6 +333,7 @@ bool SetUpDBM(DBM* dbm, bool writable, bool initialize, const std::string& file_
     tuning_params.fbp_capacity = fbp_cap;
     tuning_params.min_read_size = min_read_size;
     tuning_params.cache_buckets = cache_buckets;
+    tuning_params.cipher_key = cipher_key;
     tuning_params.max_page_size = max_page_size;
     tuning_params.max_branches = max_branches;
     tuning_params.max_cached_pages = max_cached_pages;
@@ -595,7 +608,7 @@ static int32_t ProcessSequence(int32_t argc, const char** args) {
     {"--sync_io", 0}, {"--padding", 0}, {"--pagecache", 0},
     {"--append", 0}, {"--record_crc", 1}, {"--record_comp", 1},
     {"--offset_width", 1}, {"--align_pow", 1}, {"--buckets", 1},
-    {"--fbp_cap", 1}, {"--min_read_size", 1}, {"--cache_buckets", 0},
+    {"--fbp_cap", 1}, {"--min_read_size", 1}, {"--cache_buckets", 0}, {"--cipher_key", 1},
     {"--max_page_size", 1}, {"--max_branches", 1}, {"--max_cached_pages", 1},
     {"--page_update_write", 0},
     {"--step_unit", 1}, {"--max_level", 1}, {"--sort_mem_size", 1}, {"--insert_in_order", 0},
@@ -643,6 +656,7 @@ static int32_t ProcessSequence(int32_t argc, const char** args) {
   const int32_t fbp_cap = GetIntegerArgument(cmd_args, "--fbp_cap", 0, -1);
   const int32_t min_read_size = GetIntegerArgument(cmd_args, "--min_read_size", 0, -1);
   const int32_t cache_buckets = CheckMap(cmd_args, "--cache_buckets") ? 1 : 0;
+  const std::string cipher_key = GetStringArgument(cmd_args, "--cipher_key", 0, "");
   const int32_t max_page_size = GetIntegerArgument(cmd_args, "--max_page_size", 0, -1);
   const int32_t max_branches = GetIntegerArgument(cmd_args, "--max_branches", 0, -1);
   const int32_t max_cached_pages = GetIntegerArgument(cmd_args, "--max_cached_pages", 0, -1);
@@ -731,7 +745,8 @@ static int32_t ProcessSequence(int32_t argc, const char** args) {
   if (is_set_only) {
     if (!SetUpDBM(dbm.get(), true, true, file_path, with_no_wait, with_no_lock, with_sync_hard,
                   is_append, record_crc, record_comp,
-                  offset_width, align_pow, num_buckets, fbp_cap, min_read_size, cache_buckets,
+                  offset_width, align_pow, num_buckets, fbp_cap, min_read_size,
+                  cache_buckets, cipher_key,
                   max_page_size, max_branches, max_cached_pages, page_update_write,
                   step_unit, max_level, sort_mem_size, insert_in_order, max_cached_records,
                   poly_params)) {
@@ -809,7 +824,8 @@ static int32_t ProcessSequence(int32_t argc, const char** args) {
   if (is_get_only) {
     if (!SetUpDBM(dbm.get(), false, false, file_path, with_no_wait, with_no_lock, with_sync_hard,
                   is_append, record_crc, record_comp,
-                  offset_width, align_pow, num_buckets, fbp_cap, min_read_size, cache_buckets,
+                  offset_width, align_pow, num_buckets, fbp_cap, min_read_size,
+                  cache_buckets, cipher_key,
                   max_page_size, max_branches, max_cached_pages, page_update_write,
                   step_unit, max_level, sort_mem_size, insert_in_order, max_cached_records,
                   poly_params)) {
@@ -883,7 +899,8 @@ static int32_t ProcessSequence(int32_t argc, const char** args) {
   if (is_iter_only) {
     if (!SetUpDBM(dbm.get(), false, false, file_path, with_no_wait, with_no_lock, with_sync_hard,
                   is_append, record_crc, record_comp,
-                  offset_width, align_pow, num_buckets, fbp_cap, min_read_size, cache_buckets,
+                  offset_width, align_pow, num_buckets, fbp_cap, min_read_size,
+                  cache_buckets, cipher_key,
                   max_page_size, max_branches, max_cached_pages, page_update_write,
                   step_unit, max_level, sort_mem_size, insert_in_order, max_cached_records,
                   poly_params)) {
@@ -944,7 +961,8 @@ static int32_t ProcessSequence(int32_t argc, const char** args) {
   if (is_remove_only) {
     if (!SetUpDBM(dbm.get(), true, false, file_path, with_no_wait, with_no_lock, with_sync_hard,
                   is_append, record_crc, record_comp,
-                  offset_width, align_pow, num_buckets, fbp_cap, min_read_size, cache_buckets,
+                  offset_width, align_pow, num_buckets, fbp_cap, min_read_size,
+                  cache_buckets, cipher_key,
                   max_page_size, max_branches, max_cached_pages, page_update_write,
                   step_unit, max_level, sort_mem_size, insert_in_order, max_cached_records,
                   poly_params)) {
@@ -1011,7 +1029,7 @@ static int32_t ProcessParallel(int32_t argc, const char** args) {
     {"--sync_io", 0}, {"--padding", 0}, {"--pagecache", 0},
     {"--append", 0}, {"--record_crc", 1}, {"--record_comp", 1},
     {"--offset_width", 1}, {"--align_pow", 1}, {"--buckets", 1},
-    {"--fbp_cap", 1}, {"--min_read_size", 1}, {"--cache_buckets", 0},
+    {"--fbp_cap", 1}, {"--min_read_size", 1}, {"--cache_buckets", 0}, {"--cipher_key", 1},
     {"--max_page_size", 1}, {"--max_branches", 1}, {"--max_cached_pages", 1},
     {"--page_update_write", 0},
     {"--step_unit", 1}, {"--max_level", 1}, {"--sort_mem_size", 1}, {"--insert_in_order", 0},
@@ -1057,6 +1075,7 @@ static int32_t ProcessParallel(int32_t argc, const char** args) {
   const int32_t fbp_cap = GetIntegerArgument(cmd_args, "--fbp_cap", 0, -1);
   const int32_t min_read_size = GetIntegerArgument(cmd_args, "--min_read_size", 0, -1);
   const int32_t cache_buckets = CheckMap(cmd_args, "--cache_buckets") ? 1 : 0;
+  const std::string cipher_key = GetStringArgument(cmd_args, "--cipher_key", 0, "");
   const int32_t max_page_size = GetIntegerArgument(cmd_args, "--max_page_size", 0, -1);
   const int32_t max_branches = GetIntegerArgument(cmd_args, "--max_branches", 0, -1);
   const int32_t max_cached_pages = GetIntegerArgument(cmd_args, "--max_cached_pages", 0, -1);
@@ -1191,7 +1210,8 @@ static int32_t ProcessParallel(int32_t argc, const char** args) {
   };
   if (!SetUpDBM(dbm.get(), true, true, file_path, with_no_wait, with_no_lock, with_sync_hard,
                 is_append, record_crc, record_comp,
-                offset_width, align_pow, num_buckets, fbp_cap, min_read_size, cache_buckets,
+                offset_width, align_pow, num_buckets, fbp_cap, min_read_size,
+                cache_buckets, cipher_key,
                 max_page_size, max_branches, max_cached_pages, page_update_write,
                 step_unit, max_level, sort_mem_size, insert_in_order, max_cached_records,
                 poly_params)) {
@@ -1256,7 +1276,7 @@ static int32_t ProcessWicked(int32_t argc, const char** args) {
     {"--sync_io", 0}, {"--padding", 0}, {"--pagecache", 0},
     {"--append", 0}, {"--record_crc", 1}, {"--record_comp", 1},
     {"--offset_width", 1}, {"--align_pow", 1}, {"--buckets", 1},
-    {"--fbp_cap", 1}, {"--min_read_size", 1}, {"--cache_buckets", 0},
+    {"--fbp_cap", 1}, {"--min_read_size", 1}, {"--cache_buckets", 0}, {"--cipher_key", 1},
     {"--max_page_size", 1}, {"--max_branches", 1}, {"--max_cached_pages", 1},
     {"--page_update_write", 0},
     {"--step_unit", 1}, {"--max_level", 1}, {"--sort_mem_size", 1}, {"--insert_in_order", 0},
@@ -1301,6 +1321,7 @@ static int32_t ProcessWicked(int32_t argc, const char** args) {
   const int32_t fbp_cap = GetIntegerArgument(cmd_args, "--fbp_cap", 0, -1);
   const int32_t min_read_size = GetIntegerArgument(cmd_args, "--min_read_size", 0, -1);
   const int32_t cache_buckets = CheckMap(cmd_args, "--cache_buckets") ? 1 : 0;
+  const std::string cipher_key = GetStringArgument(cmd_args, "--cipher_key", 0, "");
   const int32_t max_page_size = GetIntegerArgument(cmd_args, "--max_page_size", 0, -1);
   const int32_t max_branches = GetIntegerArgument(cmd_args, "--max_branches", 0, -1);
   const int32_t max_cached_pages = GetIntegerArgument(cmd_args, "--max_cached_pages", 0, -1);
@@ -1514,7 +1535,8 @@ static int32_t ProcessWicked(int32_t argc, const char** args) {
   };
   if (!SetUpDBM(dbm.get(), true, true, file_path, with_no_wait, with_no_lock, with_sync_hard,
                 is_append, record_crc, record_comp,
-                offset_width, align_pow, num_buckets, fbp_cap, min_read_size, cache_buckets,
+                offset_width, align_pow, num_buckets, fbp_cap, min_read_size,
+                cache_buckets, cipher_key,
                 max_page_size, max_branches, max_cached_pages, page_update_write,
                 step_unit, max_level, sort_mem_size, insert_in_order, max_cached_records,
                 poly_params)) {
@@ -1575,7 +1597,7 @@ static int32_t ProcessIndex(int32_t argc, const char** args) {
     {"--path", 1},
     {"--append", 0}, {"--record_crc", 1}, {"--record_comp", 1},
     {"--offset_width", 1}, {"--align_pow", 1}, {"--buckets", 1},
-    {"--fbp_cap", 1}, {"--min_read_size", 1}, {"--cache_buckets", 0},
+    {"--fbp_cap", 1}, {"--min_read_size", 1}, {"--cache_buckets", 0}, {"--cipher_key", 1},
     {"--max_page_size", 1}, {"--max_branches", 1}, {"--max_cached_pages", 1},
     {"--page_update_write", 0},
   };
@@ -1601,6 +1623,7 @@ static int32_t ProcessIndex(int32_t argc, const char** args) {
   const int32_t fbp_cap = GetIntegerArgument(cmd_args, "--fbp_cap", 0, -1);
   const int32_t min_read_size = GetIntegerArgument(cmd_args, "--min_read_size", 0, -1);
   const int32_t cache_buckets = CheckMap(cmd_args, "--cache_buckets") ? 1 : 0;
+  const std::string cipher_key = GetStringArgument(cmd_args, "--cipher_key", 0, "");
   const int32_t max_page_size = GetIntegerArgument(cmd_args, "--max_page_size", 0, -1);
   const int32_t max_branches = GetIntegerArgument(cmd_args, "--max_branches", 0, -1);
   const int32_t max_cached_pages = GetIntegerArgument(cmd_args, "--max_cached_pages", 0, -1);
