@@ -134,7 +134,7 @@ extern const char* const TKRZW_ANY_DATA;
  * of the return value is to be stored.
  */
 typedef const char* (*tkrzw_record_processor)(
-    void* arg, const char*, int32_t, const char*, int32_t, int32_t*);
+    void*, const char*, int32_t, const char*, int32_t, int32_t*);
 
 /** The special string indicating no operation. */
 extern const char* const TKRZW_REC_PROC_NOOP;
@@ -462,8 +462,9 @@ bool tkrzw_dbm_close(TkrzwDBM* dbm);
  * @param writable True if the processor can edit the record.
  * @return True on success or false on failure.
  * @details If the specified record exists, the value is given to the callback function.  If it
- * doesn't exist, NULL is given instead.  If the callback function returns TKRZW_REC_PROC_NOOP,
- * TKRZW_REC_PROC_REMOVE, or a string pointer to a new value, whose ownership is not taken.
+ * doesn't exist, NULL is given instead.  The callback function returns TKRZW_REC_PROC_NOOP to
+ * keep the current value, TKRZW_REC_PROC_REMOVE to remove the record, or a string pointer to a
+ * new value to set.  The ownership of the return value is not taken.
  */
 bool tkrzw_dbm_process(
     TkrzwDBM* dbm, const char* key_ptr, int32_t key_size, tkrzw_record_processor proc,
@@ -691,8 +692,9 @@ int64_t tkrzw_dbm_increment(
  * @param writable True if the processors can edit the records.
  * @return True on success or false on failure.
  * @details If the specified record exists, the value is given to the callback function.  If it
- * doesn't exist, NULL is given instead.  If the callback function returns TKRZW_REC_PROC_NOOP,
- * TKRZW_REC_PROC_REMOVE, or a string pointer to a new value, whose ownership is not taken.
+ * doesn't exist, NULL is given instead.  The callback function returns TKRZW_REC_PROC_NOOP
+ * to keep the current value, TKRZW_REC_PROC_REMOVE to remove the record, or a string pointer to a
+ * new value to set.  The ownership of the return value is not taken.
  */
 bool tkrzw_dbm_process_multi(
     TkrzwDBM* dbm, TkrzwKeyProcPair* key_proc_pairs, int32_t num_pairs, bool writable);
@@ -786,8 +788,10 @@ bool tkrzw_dbm_push_last(TkrzwDBM* dbm, const char* value_ptr, int32_t value_siz
  * @param proc_arg An arbitrary data which is given to the callback function.
  * @param writable True if the processor can edit the record.
  * @details If the specified record exists, the value is given to the callback function.  If it
- * doesn't exist, NULL is given instead.  If the callback function returns TKRZW_REC_PROC_NOOP,
- * TKRZW_REC_PROC_REMOVE, or a string pointer to a new value, whose ownership is not taken.
+ * doesn't exist, NULL is given instead.  The callback function returns TKRZW_REC_PROC_NOOP to
+ * keep the current value, TKRZW_REC_PROC_REMOVE to remove the record, or a string pointer to a
+ * new value to set.  The ownership of the return value is not taken.  It is also called once
+ * before the iteration and once after the iteration with both the key and the value being NULL.
  */
 bool tkrzw_dbm_process_each(
     TkrzwDBM* dbm, tkrzw_record_processor proc, void* proc_arg, bool writable);
@@ -939,9 +943,13 @@ bool tkrzw_dbm_is_ordered(TkrzwDBM* dbm);
  * @param mode The search mode.  "contain" extracts keys containing the pattern.  "begin"
  * extracts keys beginning with the pattern.  "end" extracts keys ending with the pattern.
  * "regex" extracts keys partially matches the pattern of a regular expression.  "edit"
- * extracts keys whose edit distance to the pattern is the least.  Ordered databases support
- * "upper" and "lower" which extract keys whose positions are equal to or upper/lower than the
- * pattern.  "upperex" and "lowerex" are their exclusive versions.
+ * extracts keys whose edit distance to the pattern is the least.  "editbin" extracts keys whose
+ * edit distance to the binary pattern is the least.  "containcase", "containword", and
+ * "containcaseword" extract keys considering case and word boundary.  "contain*", "containcase*",
+ * "containword*", and "containcaseword*" take a null-code-separatable pattern and do batch
+ * operations for each element.  Ordered databases support "upper" and "lower" which extract keys
+ * whose positions are equal to or upper/lower than the pattern.  "upperex" and "lowerex" are
+ * their exclusive versions.
  * @param pattern_ptr The pattern pointer.
  * @param pattern_size The pattern size.  If it is negative, strlen(pattern_ptr) is used.
  * @param capacity The maximum records to obtain.  0 means unlimited.
@@ -1048,10 +1056,10 @@ bool tkrzw_dbm_iter_previous(TkrzwDBMIter* iter);
  * @param writable True if the processor can edit the record.
  * @return True on success or false on failure.
  * @details If the current record exists, the callback function is called.  Otherwise, this
- * method fails and the callback is not called.  If the callback function returns
- * TKRZW_REC_PROC_NOOP, TKRZW_REC_PROC_REMOVE, or a string pointer to a new value, whose
- * ownership is not taken.  If the current record is removed, the iterator is moved to the next
- * record.
+ * method fails and the callback is not called.  The callback function returns TKRZW_REC_PROC_NOOP
+ * to keep the current value, TKRZW_REC_PROC_REMOVE to remove the record, or a string pointer to a
+ * new value to set.  The ownership of the return value is not taken.  If the current record is
+ * removed, the iterator is moved to the next record.
  */
 bool tkrzw_dbm_iter_process(
     TkrzwDBMIter* iter, tkrzw_record_processor proc, void* proc_arg, bool writable);
@@ -1137,11 +1145,13 @@ bool tkrzw_dbm_iter_step(
  * @param end_offset The exclusive end offset of records to read.  Negative means unlimited.
  * 0 means the size when the database is synched or closed properly.  Using a positive value
  * is not meaningful if the number of shards is more than one.
+ * @param cipher_key The encryption key for cipher compressors.  If it is NULL, an empty key is
+ * used.
  * @return True on success or false on failure.
  */
 bool tkrzw_dbm_restore_database(
     const char* old_file_path, const char* new_file_path,
-    const char* class_name, int64_t end_offset);
+    const char* class_name, int64_t end_offset, const char* cipher_key);
 
 /**
  * Creates an asynchronous database adapter.
