@@ -399,6 +399,7 @@ TEST(StrUtilTest, StrWordContains) {
   EXPECT_FALSE(tkrzw::StrWordContains("hop step jump", "ump"));
   EXPECT_FALSE(tkrzw::StrWordContains("hop step jump", "jumper"));
   EXPECT_FALSE(tkrzw::StrWordContains("hop step jump", "p s"));
+  EXPECT_TRUE(tkrzw::StrWordContains("hop step jump st", "st"));
 }
 
 TEST(StrUtilTest, StrCaseWordContains) {
@@ -415,6 +416,77 @@ TEST(StrUtilTest, StrCaseWordContains) {
   EXPECT_FALSE(tkrzw::StrCaseWordContains("hOp sTep jumP", "ump"));
   EXPECT_FALSE(tkrzw::StrCaseWordContains("hOp sTep jumP", "jumper"));
   EXPECT_FALSE(tkrzw::StrCaseWordContains("hOp sTep jumP", "p s"));
+  EXPECT_TRUE(tkrzw::StrCaseWordContains("hOp sTep jumP ST", "st"));
+}
+
+TEST(StrUtilTest, StrContainsBatch) {
+  EXPECT_FALSE(tkrzw::StrContainsBatch("", std::vector<std::string>({})));
+  EXPECT_TRUE(tkrzw::StrContainsBatch("", std::vector<std::string>({""})));
+  EXPECT_FALSE(tkrzw::StrContainsBatch("abcdef", std::vector<std::string>({"123"})));
+  EXPECT_TRUE(tkrzw::StrContainsBatch("abcdef", std::vector<std::string>({"123", "abc"})));
+  EXPECT_TRUE(tkrzw::StrContainsBatch("abcdef", std::vector<std::string>({"bcd", "123"})));
+}
+
+TEST(StrUtilTest, StrCaseContainsBatch) {
+  EXPECT_FALSE(tkrzw::StrCaseContainsBatch("", std::vector<std::string>({})));
+  EXPECT_TRUE(tkrzw::StrCaseContainsBatch("", std::vector<std::string>({""})));
+  EXPECT_FALSE(tkrzw::StrCaseContainsBatch("AbCdEf", std::vector<std::string>({"123"})));
+  EXPECT_TRUE(tkrzw::StrCaseContainsBatch("AbCdEf", std::vector<std::string>({"123", "aBc"})));
+  EXPECT_TRUE(tkrzw::StrCaseContainsBatch("AbCdEf", std::vector<std::string>({"BcD", "123"})));
+}
+
+TEST(StrUtilTest, StrWordContainsBatch) {
+  EXPECT_FALSE(tkrzw::StrWordContainsBatch("", std::vector<std::string>({})));
+  EXPECT_TRUE(tkrzw::StrWordContainsBatch("", std::vector<std::string>({""})));
+  EXPECT_FALSE(tkrzw::StrWordContainsBatch("abc def", std::vector<std::string>({"123"})));
+  EXPECT_TRUE(tkrzw::StrWordContainsBatch("abc def", std::vector<std::string>({"123", "abc"})));
+  EXPECT_TRUE(tkrzw::StrWordContainsBatch("abc def", std::vector<std::string>({"123", "def"})));
+  EXPECT_TRUE(tkrzw::StrWordContainsBatch("abc def", std::vector<std::string>({"abc def"})));
+  EXPECT_FALSE(tkrzw::StrWordContainsBatch("abc def", std::vector<std::string>({"bc", "123"})));
+  EXPECT_FALSE(tkrzw::StrWordContainsBatch("abc def", std::vector<std::string>({"ab", "123"})));
+  EXPECT_TRUE(tkrzw::StrWordContainsBatch("abc def ab", std::vector<std::string>({"ab", "123"})));
+}
+
+TEST(StrUtilTest, StrCaseWordContainsBatch) {
+  EXPECT_FALSE(tkrzw::StrCaseWordContainsBatch("", std::vector<std::string>({})));
+  EXPECT_TRUE(tkrzw::StrCaseWordContainsBatch("", std::vector<std::string>({""})));
+  EXPECT_FALSE(tkrzw::StrCaseWordContainsBatch(
+      "aBc DeF", std::vector<std::string>({"123"})));
+  EXPECT_TRUE(tkrzw::StrCaseWordContainsBatch(
+      "aBc DeF", std::vector<std::string>({"123", "AbC"})));
+  EXPECT_TRUE(tkrzw::StrCaseWordContainsBatch(
+      "aBc DeF", std::vector<std::string>({"123", "dEf"})));
+  EXPECT_TRUE(tkrzw::StrCaseWordContainsBatch(
+      "aBc DeF", std::vector<std::string>({"AbC def"})));
+  EXPECT_FALSE(tkrzw::StrCaseWordContainsBatch(
+      "aBc DeF", std::vector<std::string>({"bc", "123"})));
+  EXPECT_FALSE(tkrzw::StrCaseWordContainsBatch(
+      "aBc DeF", std::vector<std::string>({"ab", "123"})));
+  EXPECT_TRUE(tkrzw::StrCaseWordContainsBatch(
+      "aBc DeF Ab", std::vector<std::string>({"AB", "123"})));
+}
+
+TEST(StrUtilTest, StrCaseWordContainsBatchLower) {
+  EXPECT_FALSE(tkrzw::StrCaseWordContainsBatchLower("", std::vector<std::string>({})));
+  EXPECT_TRUE(tkrzw::StrCaseWordContainsBatchLower("", std::vector<std::string>({""})));
+  EXPECT_FALSE(tkrzw::StrCaseWordContainsBatchLower(
+      "aBc DeF", std::vector<std::string>({"123"})));
+  EXPECT_TRUE(tkrzw::StrCaseWordContainsBatchLower(
+      "aBc DeF", std::vector<std::string>({"123", "abc"})));
+  EXPECT_TRUE(tkrzw::StrCaseWordContainsBatchLower(
+      "aBc DeF", std::vector<std::string>({"123", "def"})));
+  EXPECT_TRUE(tkrzw::StrCaseWordContainsBatchLower(
+      "aBc DeF", std::vector<std::string>({"abc def"})));
+  EXPECT_FALSE(tkrzw::StrCaseWordContainsBatchLower(
+      "aBc DeF", std::vector<std::string>({"bc", "123"})));
+  EXPECT_FALSE(tkrzw::StrCaseWordContainsBatchLower(
+      "aBc DeF", std::vector<std::string>({"ab", "123"})));
+  EXPECT_TRUE(tkrzw::StrCaseWordContainsBatchLower(
+      "aBc DeF Ab", std::vector<std::string>({"ab", "123"})));
+  EXPECT_TRUE(tkrzw::StrCaseWordContainsBatchLower(
+      "aBc DeF Ab", std::vector<std::string>({"ab", "123"})));
+  EXPECT_TRUE(tkrzw::StrCaseWordContainsBatchLower(
+      "aBc DeF Ab-", std::vector<std::string>({"ab", "123"})));
 }
 
 TEST(StrUtilTest, StrBeginsWith) {
